import { Injectable } from '@nestjs/common';
import { InjectEntityManager, InjectRepository } from '@nestjs/typeorm';
import { DomainEntity } from './domain.entity';
import { EntityManager, Like, Not, Repository } from 'typeorm';
import { DomainDto, DomainQueryDto, DomainUpdateDto } from './domain.dto';
import { Pagination } from '~/helper/paginate/pagination';
import { isNumber } from 'lodash';
import { paginate } from '~/helper/paginate';
import { Storage } from '../tools/storage/storage.entity';
import { BusinessException } from '~/common/exceptions/biz.exception';
import { ErrorEnum } from '~/constants/error-code.constant';
import { fieldSearch } from '~/shared/database/field-search';

@Injectable()
export class DomainService {
  constructor(
    @InjectEntityManager() private entityManager: EntityManager,
    @InjectRepository(DomainEntity)
    private domainRepository: Repository<DomainEntity>
  ) {}

  /**
   * 查找所有域
   */
  async findAll({ page, pageSize, ...fields }: DomainQueryDto): Promise<Pagination<DomainEntity>> {
    const queryBuilder = this.domainRepository
      .createQueryBuilder('domain')
      .where(fieldSearch(fields))
      .andWhere('domain.isDelete = 0');

    return paginate<DomainEntity>(queryBuilder, {
      page,
      pageSize
    });
  }

  /**
   * 新增
   */
  async create({ title, ...ext }: DomainDto): Promise<void> {
    if (await this.checkIsDomainExsit(title)) {
      throw new BusinessException(ErrorEnum.DOMAIN_TITLE_DUPLICATE);
    }
    await this.domainRepository.insert(this.domainRepository.create({ title, ...ext }));
  }

  /**
   * 更新
   */
  async update(id: number, { title, ...ext }: Partial<DomainUpdateDto>): Promise<void> {
    await this.entityManager.transaction(async manager => {
      if (title && (await this.checkIsDomainExsit(title, id))) {
        throw new BusinessException(ErrorEnum.CONTRACT_NUMBER_EXIST);
      }
      await manager.update(DomainEntity, id, {
        ...ext,
        title
      });
    });
  }

  /**
   * 是否存在相同的域
   * @param title 域编号
   */
  async checkIsDomainExsit(title: string, id?: number): Promise<Boolean> {
    return !!(await this.domainRepository.findOne({
      where: {
        title: title,
        id: Not(id)
      }
    }));
  }
  /**
   * 删除
   */
  async delete(id: number): Promise<void> {
    // 域比较重要，做逻辑删除
    await this.domainRepository.update(id, { isDelete: 1 });
  }

  /**
   * 获取单个域信息
   */
  async info(id: number) {
    const info = await this.domainRepository
      .createQueryBuilder('domain')
      .where({
        id
      })
      .andWhere('domain.isDelete = 0')
      .getOne();
    return info;
  }
}
