import { Body, Controller, Delete, Get, Post, Put, Query, Res } from '@nestjs/common';
import { ApiOperation, ApiTags } from '@nestjs/swagger';
import { ApiResult } from '~/common/decorators/api-result.decorator';
import { IdParam } from '~/common/decorators/id-param.decorator';
import { MaterialsInOutService } from './materials_in_out.service';
import { MaterialsInOutEntity } from './materials_in_out.entity';
import { ApiSecurityAuth } from '~/common/decorators/swagger.decorator';
import { definePermission, Perm } from '~/modules/auth/decorators/permission.decorator';
import {
  MaterialsInOutQueryDto,
  MaterialsInOutDto,
  MaterialsInOutUpdateDto,
  MaterialsInOutExportDto
} from './materials_in_out.dto';
import { Domain, DomainType, SkDomain } from '~/common/decorators/domain.decorator';
import { FastifyReply } from 'fastify';
export const permissions = definePermission('materials_inventory:history_in_out', {
  LIST: 'list',
  CREATE: 'create',
  READ: 'read',
  UPDATE: 'update',
  DELETE: 'delete',
  EXPORT: 'export'
} as const);

@ApiTags('Materials In Out History - 原材料出入库记录')
@ApiSecurityAuth()
@Controller('materials-in-out')
export class MaterialsInOutController {
  constructor(private materialsInOutService: MaterialsInOutService) { }

  @Get('export')
  @ApiOperation({ summary: '导出原材料盘点表' })
  @Perm(permissions.EXPORT)
  async exportMaterialsInventoryCheck(
    @Domain() domain: SkDomain,
    @Query() dto: MaterialsInOutExportDto,
    @Res() res: FastifyReply
  ): Promise<void> {
    await this.materialsInOutService.exportMaterialsInventoryCheck({ ...dto, domain }, res);
  }


  @Get()
  @ApiOperation({ summary: '获取原材料出入库记录列表' })
  @ApiResult({ type: [MaterialsInOutEntity], isPage: true })
  @Perm(permissions.LIST)
  async list(@Domain() domain: SkDomain, @Query() dto: MaterialsInOutQueryDto) {
    return this.materialsInOutService.findAll({ ...dto, domain });
  }

  @Get(':id')
  @ApiOperation({ summary: '获取原材料出入库记录信息' })
  @ApiResult({ type: MaterialsInOutDto })
  @Perm(permissions.READ)
  async info(@IdParam() id: number) {
    return this.materialsInOutService.info(id);
  }

  @Post()
  @ApiOperation({ summary: '新增原材料出入库记录' })
  @Perm(permissions.CREATE)
  async create(@Domain() domain: SkDomain, @Body() dto: MaterialsInOutDto): Promise<number> {
    return this.materialsInOutService.create({ ...dto, domain });
  }

  @Put(':id')
  @ApiOperation({ summary: '更新原材料出入库记录' })
  @Perm(permissions.UPDATE)
  async update(@IdParam() id: number, @Body() dto: MaterialsInOutUpdateDto): Promise<void> {
    await this.materialsInOutService.update(id, dto);
  }

  @Delete(':id')
  @ApiOperation({ summary: '删除原材料出入库记录' })
  @Perm(permissions.DELETE)
  async delete(@IdParam() id: number): Promise<void> {
    await this.materialsInOutService.delete(id);
  }

  @Put('unlink-attachments/:id')
  @ApiOperation({ summary: '附件解除关联' })
  @Perm(permissions.UPDATE)
  async unlinkAttachments(
    @IdParam() id: number,
    @Body() { fileIds }: MaterialsInOutUpdateDto
  ): Promise<void> {
    await this.materialsInOutService.unlinkAttachments(id, fileIds);
  }
}
